import numpy as np
import pylab as pl

import util
import data_atom_ionization as D

red = (.9, 0., 0.1)
green = (0., 0.9, 0.2)
blue = (.2, 0., .8)

def ADK(F, E0, Z=1, nelectrons=2, l=0):
    if l == 0:
        lfactor = 3.0
    elif l == 1:
        lfactor = 1.0

    nstar = Z / np.sqrt(2.0 * E0)
    b1 = 3. * np.exp(1) / (2.0 * np.pi) 
    b2 = Z**2 / nstar**4
    b3 = (4 * np.exp(1) * Z**3 / (nstar**4 * F))**(2.*nstar-1)
    b4 = np.exp(-2.*Z**3 / (3.*nstar**3*F))
    return nelectrons * b1 * b2 * b3 * b4 / lfactor


fig = pl.figure(figsize=(util.goodfigwidth, util.goodfigwidth * .52))
Hefig = pl.figure(figsize=(util.goodfigwidth, util.goodfigwidth * .52))

for fig1 in [fig, Hefig]:
    fig1.subplots_adjust(left=.15, right=.99, top=.96, bottom=.2)


ax = fig.add_subplot(111)
Heax = Hefig.add_subplot(111)


def cat(a1, a2):
    return np.concatenate([a1, a2])


EHepts = list(D.EHe_oblong[2:4]) + list(D.EHe_samplemore) + list(D.EHe_oblong[-5:])
GHepts = list(D.GHe_oblong[2:4]) + list(D.GHe_samplemore) + list(D.GHe_oblong[-5:])

Heax.plot(EHepts, np.array(GHepts), '-', color=red, label='He LDA', zorder=4)

G2 = cat(D.GHe_hh_samplemore, D.GHe_oblong_hh[8:])
Heax.plot(cat(D.EHe_hh_samplemore, D.EHe_oblong_hh[8:]),
          G2, color=blue, dashes=(5, 2), label='He EXX', zorder=3)

def Iwcm22au(I):
    return np.array(I) / 3.50944758e16

def ItoE(I):
    return np.sqrt(I / 137. / 1. * 2 * 4. * np.pi)

def Isec2au(G):
    return np.array(G) * 2.418884326505e-17

#Herefhh = D.GHe_oblong_hh[0]
Heax.plot(D.EHe_scrinzi2, D.GHe_scrinzi2, color=green, dashes=(3, 1, 1, 1),
          label='He Scrinzi', zorder=3)

xadk = np.linspace(0.0, 0.5, 200)
Heax.plot(xadk, ADK(xadk, 24.5874 / 27.211), 
          color='m', dashes=(1, 1),
          label='He ADK', zorder=-3)

#Heax.plot(xadk, ADK(xadk, .585),
#          color='c', dashes=(3, 1, 1, 1),
#          label='He ADK/LDA', zorder=-3)
#Heax.plot(xadk, ADK(xadk, .925), <-- this one is very good actually
#          color='y', dashes=(3, 1, 1, 1),
#          label='He ADK/EXX', zorder=-3)


eionLi = 5.3917 / 27.211
nstarLi = 1.0 / np.sqrt(2.0 * eionLi)
CnlLi = (2.0 * np.exp(1) / nstarLi)**nstarLi / np.sqrt(2.0 * np.pi * nstarLi)

XLi = 2.0 * (2.0 * eionLi)**(3./2.) / xadk

WADKLi = CnlLi**2 * eionLi * XLi**(2.0 * nstarLi - 1) * np.exp(-XLi / 3)
# agrees with ours
#ax.plot(xadk, WADKLi, color='c', ls='-.', label='grumble')

# LDA PLOTS
ax.plot(D.EH_better, D.GH_better, '-', color=red, label='H LDA', zorder=4)
ax.plot(cat(D.ELi_veryserious[1:], D.ELi_bigcell[4:]), 
        cat(D.GLi_veryserious[1:], D.GLi_bigcell[4:]),
        '-', color=blue, label='Li LDA', zorder=4)
NBe = 4
LBe = ax.plot(cat(D.EBe_fine, D.EBe_bigcell[NBe:]), 
              cat(D.GBe_fine, D.GBe_bigcell[NBe:]),
              '-', color=green, label='Be LDA', zorder=4)

# REF PLOTS
#ax.plot(D.ELi_meng, D.GLi_meng, 'cs-', #dashes=(3,1,1,1), 
#        label='Li Meng')
ax.plot(D.EH_independent[2:], D.GH_independent[2:], '-', color=red,
        label='H Sch.Eq.', zorder=3, dashes=(3, 1, 1, 1))
ax.plot(D.ELi_greek, D.GLi_greek, '-', label='Li ref', zorder=3,
        dashes=(3,1,1,1), color=blue)
ax.plot(D.EBe_greek, D.GBe_greek, '-', label='Be ref', zorder=3,
        dashes=(3,1,1,1), color=green)

#Beref = D.GBe_fine[0]
#ax.plot(cat(D.EBe_fine, D.EBe_bigcell[NBe:]), 
#        cat(D.GBe_fine, D.GBe_bigcell[NBe:]) - Beref,
#        'go-', label='Be LDA')

# ADK PLOTS

ax.plot(xadk, ADK(xadk, 0.5, nelectrons=1), color=red, dashes=(1,1),
        label='H ADK', zorder=-3)

# KS H
#ax.plot(xadk, ADK(xadk, 6.35386/27.211, nelectrons=1), color='b', 
#        dashes=(1,1))

ax.plot(xadk, ADK(xadk, 5.391719 / 27.211, nelectrons=1),
        color=blue, dashes=(1, 1), label='Li ADK', zorder=-3)

# KS Li
#ax.plot(xadk, ADK(xadk, 2.86585/27.211, nelectrons=1),
#        color='y', dashes=(1,1))

ax.plot(xadk, ADK(xadk, 9.32266 / 27.211),
        color=green, dashes=(1, 1),
        label='Be ADK', zorder=-3)

# KS Be
#ax.plot(xadk, ADK(xadk, 5.60050 / 27.211),
#        'k--', dashes=(1, 1),
#        )

# KS He
#Heax.plot(xadk, ADK(xadk, 15.51920/27.211),
#          'k--', dashes=(3, 1, 1, 1),
#          )

#Eunit = r'$\textrm{a.u.} = 5.14\times10^{11}$ V/m'
Eunit = 'a.u.'
Gunit = 'a.u.'

for ax0 in [ax, Heax]:
    figid = {ax: 'b)', Heax: 'a)'}[ax0]
    ax0.set_ylabel(util.txt('%s Ionization rate [%s]' % (figid, Eunit)))
    ax0.set_xlabel(util.txt('Field strength [%s]' % Gunit))
    ax0.set_yscale('log')
    ax0.grid()

Heax.legend(loc='lower right', numpoints=1)

labels = ['H LDA',
          'Li LDA',
          'Be LDA',
          'H S.E.', 'Li Ref', 'Be Ref', 'H ADK', 'Li ADK', 'Be ADK']

ax.legend(labels, handlelength=1.4,
          loc='lower right', numpoints=1,
          ncol=3, columnspacing=0.2, handletextpad=0.1)

if 0:
    P1 = ax.plot([], color='k', ls='-', marker='o', label='LDA')
    P2 = ax.plot([], color='k', ls='-', marker='s', label='Ref')
    P3 = ax.plot([], color='k', ls=':', label='ADK')
    L1 = ax.legend([P1, P2, P3], 'LDA Ref ADK'.split(),
                   #loc='lower center', 
                   loc='lower right',
                   bbox_to_anchor=(.8, .01),
                   title='Method')

    PJ = []
    symbols = 'H Li Be'.split()
    for i, sym in enumerate(symbols):
        PJ.append(ax.plot([], color='rcg'[i], label=util.txt(sym)))

    #PJ.reverse()
    #fields.reverse()
    L2 = ax.legend(PJ, symbols, #loc=(1.0 - 0.05, 0.01), 
                   loc='lower right',
                   bbox_to_anchor=(.99, .01), title='Atom')
    ax.add_artist(L1)


#ax.legend(loc='lower right', ncol=2, columnspacing=1.0,
#          handletextpad=0.24)

ax.axis(xmax=0.13, ymin=1e-6, ymax=1)
Heax.axis(ymin=1e-6, ymax=1.0, xmin=0.05, xmax=.48)

util.plsavefig(fig, 'ionization-atoms')
util.plsavefig(Hefig, 'ionization-He')
util.plshow()
