import numpy as np
import pylab as pl
import util

fig = pl.figure(figsize=(util.goodfigwidth * .6, util.goodfigwidth * .57))

#ax = fig.add_subplot(111)

numbers = []
for line in open('zwf.txt'):
    if line.startswith('#'):
        continue
    if line.isspace() or len(line) == 0:
        continue
    numbers.append(map(float, line.split()))

numbers = np.array(numbers)
x, y, Re, Im = numbers.T
Re *= -1
Im *= -1

dx = 0.1
dy = 0.1
xv = np.arange(-13.0, 9.7, dx)
yv = np.arange(-10.0, 10.0, dx)

X, Y = np.meshgrid(xv, yv)

iRe = np.zeros((len(xv), len(yv)))
iIm = np.zeros((len(xv), len(yv)))
iAbs = np.zeros((len(xv), len(yv)))
iarg = np.zeros((len(xv), len(yv)))

#A = np.abs(Re + 1j * Im)
#logR = np.log(Re + 1j * Im)



for i, (x0, y0, Re0, Im0) in enumerate(zip(x, y, Re, Im)):
    i1 = np.round((x0 - xv[0]) / dx) 
    i2 = np.round((y0 - yv[0])/ dy)
    iRe[i1, i2] = Re0#A0
    iIm[i1, i2] = Im0
    z = np.log(Re0 + 1j * Im0)
    iAbs[i1, i2] = z.real
    
    iarg[i1, i2] = z.imag


xmin1 = 80
xmax1 = -50
ymin1 = 70
ymax1 = -70

#fig2 = pl.figure()
#axq = fig2.add_subplot(111)
#axq.plot(Re0)
#axq.plot(Im0)
#pl.show()

fig.subplots_adjust(top=.98, left=.1, right=.88, bottom=.02, hspace=.02)

#fig.text(.1+0.15, .98-0.05, 'hello', color='w', size=8, 
#          bbox=dict(facecolor='k', alpha=.5))

#targets = [iRe, iIm, iAbs, iarg]
targets = [iRe, iIm]
fig.text(0.03, 0.5, r'Complex-scaled\,\, wavefunction\,\, of\,\, H$_2$', 
         rotation=90,
         size=7,
         horizontalalignment='left', verticalalignment='center')
for i, R in enumerate(targets):
    ax = fig.add_subplot(2, 1, i + 1)
    
    #ax = fig.add_subplot(2, 2, i + 1)
    #ax.set_xticks([-4, -2, 0, 2, 4])
    if i == 0:
        #ax.set_xticklabels([''] * len(ax.get_xticklabels()))
        vmin = 0.0
        vmax = .45
        cticks = [0.0, .1, .2, .3, .4]
    else:
        #ax.set_xlabel(r'$x \textrm{ [a.u.]}$')
        cticks = [0., .05, .1, .15]
        vmin = -.01
        vmax = .15
    #ax.matshow(R[xmin:xmax, ymin:ymax])
    
    image = ax.matshow(R[xmin1:xmax1, ymin1:ymax1].T, origin='lower',
                      vmin=vmin, vmax=vmax,
                      #interpolation='bicu',
                      extent=(xv[xmin1], xv[xmax1], yv[ymin1], yv[ymax1]))
    #ax.set_ylabel(r'$y \textrm{ [a.u.]}$')
    for func in ax.set_xticks, ax.set_yticks:
        func([])
               

    figspec = [r'$\textrm{Re }\psi^\theta(\mathbf r)$', 
               r'$\textrm{Im }\psi^\theta(\mathbf r)$'][i]

    xmin = -5.
    xmax = 3.5
    ymax = 2.5
    ymin = -2.5

    if i == 0:
        Dx = 1
        ax.plot([xmin + Dx, xmin + 1. + Dx], [ymin + .5] * 2, color='w')
        #ax.arrow(xmin + .5, ymin + .5, 1.0, 0.0,
        #         color='w')
        ax.text(xmin + Dx + .5, ymin + .8, '1 Bohr',
                color='w', size=6,
                horizontalalignment='center',
                verticalalignment='bottom')

    #ax.axhline(0.0, ls=':', color='w')
    #ax.axvline(-
    #ax.plot([-0.69652086285129611, 0.69652086285129611], [0., 0.],
    #        'wx')


    ax.text(xmin + .4, ymax - .4, figspec, color='w',
             horizontalalignment='left',
             verticalalignment='top',
             size=7,
             bbox=dict(facecolor='k', alpha=1.))
    size = 6
    if i == 0:
        pass
        #ax.text(xmin + 0.4, -2.0, r'Wavefunction of H$_2$ molecule',
        #        color='white', size=size,
        #        horizontalalignment='left')
                
        #connection = "angle3,angleA=0,angleB=-90"
        #ax.annotate('H$_2$ molecule',
        #            xy=(-0.1, -0.2),
        #            xytext=(xmin + 0.4, -2.0),
        #            arrowprops=dict(arrowstyle='-|>',
        #                            connectionstyle=connection,
        #                            color='white'),
        #            horizontalalignment='left',
        #            size=size,
        #            color='white')
    if i == 1:
        ax.annotate('Electric field direction', 
                    xy=(xmax - .3, -1.8),  # point
                    xytext=(xmin + .4, -1.8), # text
                    color='white',
                    size=size,
                    arrowprops=dict(arrowstyle="->",
                                    color='w'),
                    horizontalalignment='left',
                    verticalalignment='center')
    ax.axis(ymax=ymax, ymin=ymin, xmin=xmin, xmax=xmax)
    pl.colorbar(image, fraction=0.08, shrink=0.85, aspect=13,
                ticks=cticks)

    
    
    #ax.set_title('hello', size=8)
#Z = np.zeros((len(x), len(y)))

#pl.plot(x)
#pl.plot(y)

util.plsavefig(fig, 'complex_density', dpi=400,
               transparent=True) #transparent=True,
               #facecolor=(255./255.,243./255.,201./255.),
               #edgecolor=(255./255.,243./255.,201./255.))
#util.plshow()
