import numpy as np
import pylab as pl
import util

import data_dissociation as D

def cat(arr1, arr2):
    return np.concatenate([arr1, arr2])

fig = pl.figure(figsize=(util.goodfigwidth, util.goodfigwidth * .6),
                dpi=150)
fig.subplots_adjust(left=.15, bottom=.16, right=.97, top=.97)
ax = fig.add_subplot(111)
#axesdim = (.16, .16, 1-.16-.05, 1-.16-.05)
#background = fig.add_axes(axesdim)
#ax = fig.add_axes(axesdim)
#assert not background is ax

markersize = 3.3

img = pl.imread('saenzdata.png')

R = img[:, :, 0]
G = img[:, :, 1]
B = img[:, :, 2]
Rr = R.ravel()

Y, X = np.indices(R.shape)

red = (R > .5) & (G < .5) & (B < .5)
green = (R < .5) & (G > .5) & (B < .5)
blue = (R < .5) & (G < .5) & (B > .5)
magenta = (R > .5) & (G < .5) & (B > .5)
cyan = (R < .5) & (G > .5) & (B > .5)


seconds_per_au = 2.418884326505e-17
saenzfactor = 1e15 * seconds_per_au

#colors = [(0., 0., .3),
#          (.5, 0., .4),
#          (1., 0.2, 0.), (1., .6, 0.), 'm']

colors = [(.8, 0., 0.1),
          (1., .7, 0.),
          (0., .9, 0.2), 
          (0.2, 0., .9), 'm']


#colors = 'bgrcm'
zorder = 20

dashes_lda = (3, 1, 1, 1)
dashes_exx = (1, 2)
dashes_saenz = []#None#(1, 0)

for i, mask in enumerate([red, magenta, blue, cyan, green]):
    color = colors[i]
    Xv = X[mask]
    Yv = Y[mask]

    args = np.argsort(Xv)
    Xv = Xv[args]
    Yv = Yv[args]

    x = Xv * 10.0 / R.shape[1]
    ymin = np.log10(2e-5)
    ymax = np.log10(1e1)
    y = ymin + (R.shape[0] - Yv) * (ymax - ymin) / R.shape[0]

    if color == 'm':
        continue
    ax.plot(x, saenzfactor * 10**y, dashes=dashes_saenz, color=color,
            zorder=zorder)

#img = pl.imread('extracted-graph.png')
#background.imshow(img, extent=(0.0, 8.0, np.log10(1e-4)-0.02, np.log10(10)))


ax.plot(D.B_notrifling_00533[:-2], D.G_notrifling_00533[:-2], 
        'o', color=colors[0], zorder=zorder, dashes=dashes_lda,
        markersize=markersize)


ax.plot(cat(D.B_notrifling_hh_00533_veryserious[:5], 
            D.B_notrifling_hh_00533[6:-2]),
        cat(D.G_notrifling_hh_00533_veryserious[:5],
            D.G_notrifling_hh_00533[6:-2]),
        'v', dashes=dashes_exx, color=colors[0], zorder=zorder,
        markersize=markersize)

ax.plot(cat(D.B_he_st_short[:-2], D.B_he_st[:-5]),
        cat(D.G_he_st_short[:-2], D.G_he_st[:-5]),
        'o', color=colors[1], zorder=zorder, dashes=dashes_lda,
        markersize=markersize)
ax.plot(cat(D.B_notrifling_hh_006_veryserious[:], D.B_he_st_hh[1:]),
        cat(D.G_notrifling_hh_006_veryserious[:], D.G_he_st_hh[1:]),
        'v', dashes=dashes_exx, color=colors[1], zorder=zorder,
        markersize=markersize)

ax.plot(D.B_notrifling_008, D.G_notrifling_008, 
        'o', dashes=dashes_lda, color=colors[2], zorder=zorder,
        markersize=markersize)

ax.plot(D.B_notrifling_hh_008, D.G_notrifling_hh_008, 
        'v', dashes=dashes_exx, color=colors[2], zorder=zorder,
        markersize=markersize)

ax.plot(D.B_notrifling_010[:-2], D.G_notrifling_010[:-2], 
        'o', dashes=dashes_lda, color=colors[3], zorder=zorder,
        markersize=markersize)
ax.plot(D.B_notrifling_hh_010, D.G_notrifling_hh_010,
        'v', dashes=dashes_exx, color=colors[3], zorder=zorder,
        markersize=markersize)


fields = '0.05338 0.06 0.08 0.10'.split()
#(2.0, 4e-5);;; 25
coords = [(1.5, 2e-6), (1.07, 1.35e-5), (1.15, 5.5e-4), (1.3, 1.75e-2)]
rotations = [50, 43, 30, 14]


P1 = ax.plot([], color='k', dashes=dashes_lda, marker='o', label='LDA',
             markersize=markersize)
P2 = ax.plot([], color='k', dashes=dashes_exx, marker='v', label='EXX',
             markersize=markersize)
P3 = ax.plot([], color='k', dashes=dashes_saenz, label='Saenz',
             markersize=markersize)
L1 = ax.legend([P1, P2, P3], 'LDA EXX Saenz'.split(), 
               loc='lower right', title='Method')

#ax.text(1.0, 1e-3, 'hello')

for i, (F, pt) in enumerate(zip(fields, coords)):
    ax.text(pt[0], pt[1], '$\mathsf{F=%s}$' % F, color=colors[i], rotation=rotations[i])
    

#PJ = []
#for i, F in enumerate(fields):
#    PJ.append(ax.plot([], color=colors[i], label=util.txt('E=%s' % F)))

#PJ.reverse()
#fields.reverse()
#L2 = ax.legend(PJ, fields, loc='lower right', title='Field strength')
#ax.add_artist(L1)

grid = ax.grid(color=(.92, .92, .92), ls='-', zorder=-50)

ax.set_yscale('log')
ax.set_xlabel(util.txt('Distance [a.u.]'))
ax.set_ylabel(util.txt('Ionization rate [a.u.]'))

ax.axis(xmin=1.0, xmax=4.0, ymin=1e-6, ymax=1e-1)

util.plsavefig(fig, 'ionization-molecule', dpi=800)
util.plshow()
