import pylab as pl
import numpy as np
import util

N = 4001
xmax = 10.0
x = np.linspace(-xmax, xmax, N)
dx = x[1] - x[0]
scalingzero = 0.0 #xmax / 2.0
Nzero = x.searchsorted(scalingzero)
err = np.abs(x[Nzero] - scalingzero)
assert err < 1e-14, (err, dx)

theta = 0.5
phase = np.exp(1j * theta)
phase2 = np.exp(2j * theta)

X = x * phase
dX = dx * phase


def stitch(potentials, startpotential, i0=0):
    v0 = np.zeros_like(potentials[0])
    v0[i0] = startpotential[i0]

    for i in range(i0 + 1, len(potentials[0])):
        errors = [abs(v[i] - v0[i - 1]) for v in potentials]
        v0[i] = (potentials[np.argmin(errors)])[i]
    
    for i in range(i0 - 1, -1, -1):
        errors = [abs(v[i] - v0[i + 1]) for v in potentials]
        v0[i] = (potentials[np.argmin(errors)])[i]
    return v0

def smartstitch(get_branch, principal, istart=0):
    currentbranch = 0
    npts = len(principal)
    g = np.zeros(npts, dtype=complex)
    g[istart] = get_branch(principal[istart], currentbranch)

    def process_point(i, prev_value, currentbranch):
        values = [get_branch(principal[i], currentbranch + adj)
                  for adj in [-1, 0, 1]]
        errors = [abs(val - prev_value) for val in values]
        adj = np.argmin(errors)
        g[i] = values[adj]
        prev_value = values[adj]
        currentbranch += [-1, 0, 1][adj]
        return prev_value, currentbranch

    # stitch from center to right
    prev_value = g[istart]
    for i in range(istart + 1, npts):
        prev_value, currentbranch = process_point(i, prev_value, currentbranch)
    
    # now stitch from center to left
    currentbranch = 0
    prev_value = g[istart]
    for i in range(istart - 1, -1, -1):
        prev_value, currentbranch = process_point(i, prev_value, currentbranch)
    return g

    

displacement = 0.0 #1.5

def fpsi(x, phase0=1.0):
    return (2./np.pi)**(.25) * np.exp(-(x * phase0 - displacement)**2)
          #(2./np.pi)**(.25) * np.exp(-(X - displacement * phase)**2)
psireal = fpsi(x)
psitheta = np.exp(1j * theta / 2.0) * fpsi(X)
nreal = psireal**2
psi = fpsi(x, phase)
Psi = psi.copy()
#Psi = fpsi(X, phase)
#Psi = (2./np.pi)**(.25) * np.exp(-(X - displacement * phase)**2)

C0I = 0.238732414637843
C1 = -0.45816529328314287
CC1 = 1.9236610509315362
CC2 = 2.5648814012420482
IF2 = 0.58482236226346462

gamma = 0.031091
alpha1 = 0.21370
beta1 = 7.5957
beta2 = 3.5876
beta3 = 1.6382
beta4 = 0.49294

def eps_c(n):
    rs = (C0I / n)**(1.0 / 3.0)
    rtrs = np.sqrt(rs)
    Q0 = -2.0 * gamma * (1.0 + alpha1 * rtrs**2)
    Q1 = 2.0 * gamma * rtrs * (beta1 +
                               rtrs * (beta2 +
                                       rtrs * (beta3 +
                                               rtrs * beta4)))
    eps_c = Q0 * np.log(1.0 + 1.0 / Q1)

    dQ1drs = gamma * (beta1 / rtrs + 2.0 * beta2 + 
                      rtrs * (3.0 * beta3 + 4.0 * beta4 * rtrs))
    dedrs = -2.0 * gamma * alpha1 * eps_c / Q0 - Q0 * dQ1drs / (Q1 * (Q1 + 1.0))

    v = eps_c - rs * dedrs / 3.0
    return eps_c, v

epscreal, vreal = eps_c(nreal)

#psitheta = np.exp(1j * theta / 2.0) * fpsi(x, phase)
#print psitheta[Nzero] / np.exp(1j * theta / 2.0), psi[Nzero], psireal[Nzero]
ntheta = psitheta**2

#print 'electrons theta', ntheta.sum() * dx

#print 'integral', (((2./np.pi)**(.25) * np.exp(-X**2))**2 * dX).sum()
assert np.abs(psi - Psi).max() < 1e-13

norm2 = (psi**2).sum() * dx# * phase
Norm2 = (Psi**2).sum() * dX
#print 'norm2', norm2
#print 'Norm2', Norm2
#print '(also, %s)' % ((psi**2).sum() * dx * phase)

psi /= norm2**.5
#Psi /= Norm2**.5

density = psi**2
Density = Psi**2

#print 'electrons', density.sum() * dx
#print 'Electrons', Density.sum() * dX

#print 'density at 0', Density[N//2], 'or', density[N//2], 'or', ntheta[N//2] / phase

def get_rtrs(n):
    #return n**(1.0 / 3.0)
    return (3.0 / (4.0 * np.pi * n))**(1.0 / 6.0)

rtrsreal = get_rtrs(nreal)
rtrs1 = get_rtrs(Density)

def get_get_branches_rootN(N):
    def get_branches_rootN(value, branch):
        return value * np.exp(2j * branch * np.pi / N)
    return get_branches_rootN
rtrs = smartstitch(get_get_branches_rootN(6), rtrs1, Nzero)

def get_branches_logarithm(value, branch):
    return value + branch * 2j * np.pi

def get_dQ1drs(rtrs):
    return gamma * (beta1 / rtrs + 2.0 * beta2 + 
                    rtrs * (3.0 * beta3 + 4.0 * beta4 * rtrs))

def get_Q0(rtrs):
    return -2.0 * gamma * (1.0 + alpha1 * rtrs**2)

def get_Q1(rtrs):
    X = rtrs * (beta1 + rtrs * (beta2 + rtrs * (beta3 + rtrs * beta4)))
    return 2.0 * gamma * X

def get_depsdrs(epsc, Q0, Q1, dQ1drs):
    return -2.0 * gamma * alpha1 * epsc / Q0 - Q0 * dQ1drs / (Q1 * (Q1 + 1.0))

dQ1drsreal = get_dQ1drs(rtrsreal)
dQ1drs = get_dQ1drs(rtrs)

Q0real = get_Q0(rtrsreal)
Q1real = get_Q1(rtrsreal)

Q0 = get_Q0(rtrs)
Q1 = get_Q1(rtrs)


tmp = 1.0 + 1.0 / Q1
principal_log_branch = np.log(tmp)
epsc = Q0 * smartstitch(get_branches_logarithm, principal_log_branch, Nzero)
epscreal = Q0real * np.log(1.0 + 1.0 / Q1real)

depsdrs = get_depsdrs(epsc, Q0, Q1, dQ1drs)
depsdrsreal = get_depsdrs(epscreal, Q0real, Q1real, dQ1drsreal)

vc_lda = epsc - rtrs**2 * depsdrs / 3.0
vc_ldareal = epscreal - rtrsreal**2 * depsdrsreal / 3.0
#print 'grrr epscreal', max(np.abs(epscreal.imag))
#print 'grrr rtrsreal', max(np.abs(rtrsreal.imag))
#print 'grrr depsdrsreal', max(np.abs(depsdrsreal.imag))
#print 'grrr', max(np.abs(vc_ldareal.imag))

Ec = (vc_lda * Density).sum() * dX
Ecreal = (vc_ldareal * nreal).sum() * dx
#print 'INT vc * n', Ec
#print 'INT vc * n real', Ecreal
#print 'Ec ERROR', abs(Ec - Ecreal)


v1 = Density**(1./3.)
v2 = Density**(1./3.) * np.exp(2j*np.pi/3.)
v3 = Density**(1./3.) * np.exp(-2j*np.pi/3.)




v0 = stitch([v1, v2, v3], v1)

w1 = stitch([v1, v2, v3], v1)
w2 = stitch([v1, v2, v3], v2)
w3 = stitch([v1, v2, v3], v3)

# The w come from "Density", not "density", and therefore we don't
# need to divide by a phase.  We just have to grab the one with
# imaginary part zero

w0index = np.argmin(np.abs(np.array([w1[N//2], w2[N//2], w3[N//2]]).imag))
w0 = [w1, w2, w3][w0index]


if 1:

    fig = pl.figure(figsize=(util.goodfigwidth * .8, 
                             util.goodfigwidth * .8 * .7),
                    dpi=150)
    fig.subplots_adjust(right=.95, top=0.75, left=0.08, bottom=.09)
    ax1 = fig.add_subplot(111)
    #ax2 = fig.add_subplot(212)

    pl.rc('font', size=14.0)

    stitchpoints = []

    for v in [v1, v2, v3]:
        for i, (q1, q2) in enumerate(zip(v[:-1], v[1:])):
            if abs(q2 - q1) > 0.01:
                v.real[i] = np.nan
                v.imag[i] = np.nan
                stitchpoints.append(i)
    dashes = (4, 1, 1, 1)
    
    legendtemplate = r'$\mathrm{%s}(v_%d)$'
    def legR(i):
        return legendtemplate % ('Re', i)
    def legI(i):
        return legendtemplate % ('Im', i)

    C1 = (0.35,0.,1.) # blue
    #C2 = (1.,0.25,0.0) # red
    #C3 = (0.,.3,0.) # green
    C2 = (1.,.25,0.0) # red
    C3 = (0.,.4,0.) # green

    lw = 1.3

    ax1.plot(x, v1.real, label=legR(1), lw=lw, ls='-', color=C1)
    ax1.plot(x, v1.imag, label=legI(1), lw=lw, dashes=dashes, color=C1)

    ax1.plot(x, v2.real, label=legR(2), lw=lw, ls='-', color=C2)
    ax1.plot(x, v2.imag, label=legI(2), lw=lw, dashes=dashes, color=C2)

    #print np.log(np.array([v1[0], v2[0], v3[0]])).imag
    ax1.plot(x, v3.real, label=legR(3), lw=lw, ls='-', color=C3)
    ax1.plot(x, v3.imag, label=legI(3), lw=lw, dashes=dashes, color=C3)
    ax1.legend(loc='lower center', bbox_to_anchor=(.5, 1.005), ncol=3)

    for i in stitchpoints:
        head_length = 0.1
        point = max([V[i - 1] for V in v1.real, v1.imag, v2.real, v2.imag,
                     v3.real, v3.imag]) + 0.12 + head_length
        arrowlength = 0.3
        ax1.arrow(x[i], point + arrowlength, 0, -arrowlength,
                  head_width=0.12, head_length=head_length,
                  fc='k', ec='k')

    G = (.8,) * 3

    ax1.plot(x, w0.real, ls='-', lw=6, zorder=0, 
             color=G, label='stitched')
    ax1.plot(x, w0.imag, lw=6, zorder=0,
             color=G, label='__nolegend__')

    ax1.set_xticks([])
    ax1.set_yticks([])
    

    for func in [util.axhline, util.axvline]:
        func(ax1)
    #ax1.grid()

    #ax2.plot(x, w0.real, ls='-', color='g', label='Re(v) stitched')
    #ax2.plot(x, w0.imag, ls='--', color='g', label='Im(v) stitched')
    #ax2.legend(loc='lower left')

    for ax in [ax1]:#, ax2]:
        #ax.axvline(0.0, ls=':', color='k')
        #ax.axhline(0.0, ls=':', color='k')
        ax.axis(xmin=-3.2, xmax=3.2, ymin=-.85, ymax=1.05)
        ax.set_ylabel(util.txt(r'Exchange potential [arb.\ units]'))
        ax.set_xlabel(util.txt(r'Distance [arb.\ units]'))
        #ax.set_xticklabels([''] * len(ax.get_xticklabels()))
        #ax.set_yticklabels([''] * len(ax.get_yticklabels()))
    #fig.savefig('stitching.pdf')
    util.plsavefig(fig, 'stitching')
    #fig.savefig('stitching.png')

#for _phase in [-1, 0, 1]:
#    print 'phase', _phase, np.exp(2j*np.pi/3. * _phase)
#    for _v in [v0, v1, v2, v3]:
#        print (_v * Density).sum() * dX * np.exp(2j*np.pi/3. * _phase)

util.plshow()
